#ifndef __CONNECTION_H_
#define __CONNECTION_H_

#include "ocicpp.h"
#include "Cursor.h"

class Connection {
private:
	OCIEnv *envhp;
	OCIServer *srvhp;
	OCIError *errhp;
	OCISvcCtx *svchp;
	OCISession *authp;
	sword status;
public:
	Connection(OCIEnv *env,const char *tnsname,const char *user,const char *password):envhp(env),srvhp(0),errhp(0),svchp(0),authp(0) {
		(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &errhp, OCI_HTYPE_ERROR, (size_t) 0, (dvoid **) 0);
		(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &srvhp, OCI_HTYPE_SERVER, (size_t) 0, (dvoid **) 0);
		(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &svchp, OCI_HTYPE_SVCCTX, (size_t) 0, (dvoid **) 0);
		CHECKERR(errhp,OCIServerAttach( srvhp, errhp, (text *)tnsname, strlen(tnsname), 0));
		(void) OCIAttrSet( (dvoid *) svchp, OCI_HTYPE_SVCCTX, (dvoid *)srvhp, (ub4) 0, OCI_ATTR_SERVER, (OCIError *) errhp);
		(void) OCIHandleAlloc((dvoid *) envhp, (dvoid **)&authp, (ub4) OCI_HTYPE_SESSION, (size_t) 0, (dvoid **) 0);
		(void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION, (dvoid *) user, (ub4) strlen(user), (ub4) OCI_ATTR_USERNAME, errhp);
		(void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION, (dvoid *) password, (ub4) strlen(password), (ub4) OCI_ATTR_PASSWORD, errhp);
		CHECKERR(errhp,OCISessionBegin ( svchp, errhp, authp, OCI_CRED_RDBMS, (ub4) OCI_DEFAULT));
		(void) OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX, (dvoid *) authp, (ub4) 0, (ub4) OCI_ATTR_SESSION, errhp);
	}
	~Connection() {
		if (errhp) {
			CHECKERR(errhp,OCIServerDetach( srvhp, errhp, OCI_DEFAULT )); 
		}
		if (srvhp) {
			CHECKERR(errhp,OCIHandleFree((dvoid *) srvhp, OCI_HTYPE_SERVER));
		}
		if (svchp) (void) OCIHandleFree((dvoid *) svchp, OCI_HTYPE_SVCCTX);
		if (errhp) (void) OCIHandleFree((dvoid *) errhp, OCI_HTYPE_ERROR);  
	} 
	Cursor *execQuery(const char *query);
	void execUpdate(const char *request);
};





#endif