
#define __OCICPP_INTERNAL_USE_
#include "Cursor.h"



Cursor::Cursor(OCIEnv *env,OCISvcCtx *svc,OCIError *err):envhp(env),svchp(svc),stmthp(0),paramd(0),
														errhp(err),defineArr(0),head(0),cell(0),
														nCols(0),nulltext(0),canFetch(0)
{
	//Alloc statement handle
	status=OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &stmthp, OCI_HTYPE_STMT, (size_t) 0, (dvoid **) 0);
	if(status!=OCI_SUCCESS) throw new OraError("Handle Alloc Failed File: "__FILE__,status);
}

sb4 Cursor::prepare(const char *sql) {
	CHECKERR(errhp,OCIStmtPrepare(stmthp, errhp,(text *) sql ,(ub4) strlen(sql), (ub4) OCI_NTV_SYNTAX, (ub4) OCI_DEFAULT));
	CHECKERR(errhp,OCIAttrGet(stmthp,OCI_HTYPE_STMT,(dvoid *) &stmtType,(ub4 *) 0,OCI_ATTR_STMT_TYPE,errhp));
	return stmtType;
}

void Cursor::exec() {
	if(stmtType==OCI_STMT_SELECT) {
		CHECKERR(errhp,OCIStmtExecute(svchp, stmthp,errhp,(ub4) 0,(ub4) 0, NULL, NULL, OCI_DEFAULT));
	} else {
		CHECKERR(errhp,OCIStmtExecute(svchp, stmthp,errhp,(ub4) 1,(ub4) 0, NULL, NULL, OCI_DEFAULT));
	}
}

void Cursor::describe() {
	CHECKERR(errhp,OCIAttrGet(stmthp,OCI_HTYPE_STMT,(dvoid *) &nCols,(ub4 *) 0,OCI_ATTR_PARAM_COUNT,errhp));
	head=new TableHead[nCols];
	cell=new RowCell[nCols];
	for(int col=0;col<nCols;col++) {
		ub2 colType;
		text *colName;
		ub4 colNameLen;
		ub2 colSize;
		ub1 prec; /* For NUMBERs only */
		CHECKERR(errhp,OCIParamGet(stmthp,OCI_HTYPE_STMT,errhp,(dvoid **) &paramd,col+1));
		CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid *) &colType,(ub4 *) 0,(ub4) OCI_ATTR_DATA_TYPE,errhp));
		CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid **) &colName,(ub4 *) &colNameLen,(ub4) OCI_ATTR_NAME,errhp));
		CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid *) &colSize,(ub4 *) 0,(ub4) OCI_ATTR_DATA_SIZE,errhp));
		if(colType==SQLT_NUM) {
			/* scale is a length of string contain number */
			CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid *) &prec, (ub4 *) 0,(ub4) OCI_ATTR_PRECISION,errhp));
			DEBUG(DLEV_DEBUG,"Precision was: %du\n",(unsigned)prec);
			colSize=(ub2)prec;
		}
		head[col].setAttrType(colType);
		head[col].setAttrName(colName,colNameLen);
		head[col].setAttrSize(colSize);
	}
}
void Cursor::define() {
	defineArr=(OCIDefine **)calloc(nCols,sizeof(OCIDefine *));
	correctTypes(); /* This is to correct e.g. NUMBER->STRING and over types what are not really usefull in host language */
	for(int i=0;i<nCols;i++) {
		cell[i].data=(dvoid *)calloc(head[i].getAttrSize()+1,sizeof(char));
		CHECKERR(errhp, OCIDefineByPos(stmthp, &defineArr[i], errhp, i+1, cell[i].data, (sword)head[i].getAttrSize(),head[i].getReqType(),
		         (dvoid *) &(cell[i].isNull), (ub2 *)&(cell[i].len), (ub2 *)0, OCI_DEFAULT));
	}	
	canFetch=1;
}	

Cursor::~Cursor() {
	delete[] cell;
	delete[] head;
	if(defineArr) free(defineArr);
	CHECKERR(errhp,OCIHandleFree(stmthp,OCI_HTYPE_STMT));
}

int Cursor::fetch() {
	if(canFetch) {
		status=OCIStmtFetch(stmthp,errhp,(ub4) 1,OCI_FETCH_NEXT,OCI_DEFAULT); 
		if(status==OCI_NO_DATA) {
			return 0;
		} else if(status==OCI_SUCCESS || status==OCI_SUCCESS_WITH_INFO) {
			/*Moved to get<Type> functions  */
			return 1;
		} else throw new OraError(errhp,status);
	} else return 0;
}
	
string *Cursor::getStr(int col) {
	if(cell[col].isNull==-1) {
		if(nulltext) return new string(nulltext);
		else return new string("null");
	} 
	if(head[col].getReqType()==SQLT_STR) return new string((char *)cell[col].data,cell[col].len);
	if(head[col].getReqType()==SQLT_AVC) return new string((char *)cell[col].data);
	if(head[col].getReqType()==SQLT_LNG) return new string((char *)cell[col].data);
	if(head[col].getReqType()==SQLT_DAT) {
		char *dat;
		char *month[]={"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};
		unsigned char cc,yy,mm,dd,hh,mi,ss;
		dat=(char *)calloc(8,1);
		cc=((unsigned char *)(cell[col].data))[0]; /* Century */
		yy=((unsigned char *)(cell[col].data))[1]; /* Year */
		mm=((char *)(cell[col].data))[2]; /* Month */
		dd=((char *)(cell[col].data))[3]; /* Day */
		hh=((char *)(cell[col].data))[4]; /* Hour */
		mi=((char *)(cell[col].data))[5]; /* Minute */
		ss=((char *)(cell[col].data))[6]; /* Seconds */
		cc=(char)abs(cc-100);
		yy=(char)abs(yy-100);
		sprintf(dat,"%d-%s-%d",dd,month[mm-1],(int)cc*100+(int)yy);
		string *retStr=new string(dat);
		free(dat);
		return retStr;
	}
}
int Cursor::getStr(int col,string *str) {
	int retVal=CELL_TYPE_UNSUPPORTED;
	if(col<0 || col>=nCols) return CELL_NOT_EXISTS;
	if(cell[col].isNull==-1) {
		if(nulltext) str->assign(nulltext);
		else str->assign("null");
		return CELL_NULL;
	}
	switch(head[col].getReqType()) {
		case SQLT_STR:
			str->assign((char *)cell[col].data,cell[col].len);
			retVal=CELL_OK;
			break;
		case SQLT_AVC:
			str->assign((char *)cell[col].data);
			retVal=CELL_OK;
			break;
		case SQLT_LNG:
			str->assign((char *)cell[col].data);
			retVal=CELL_OK;
			break;
		case SQLT_DAT:
			char *dat;
			char *month[]={"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};
			unsigned char cc,yy,mm,dd,hh,mi,ss;
			dat=(char *)calloc(8,1);
			cc=((unsigned char *)(cell[col].data))[0]; /* Century */
			yy=((unsigned char *)(cell[col].data))[1]; /* Year */
			mm=((char *)(cell[col].data))[2]; /* Month */
			dd=((char *)(cell[col].data))[3]; /* Day */
			hh=((char *)(cell[col].data))[4]; /* Hour */
			mi=((char *)(cell[col].data))[5]; /* Minute */
			ss=((char *)(cell[col].data))[6]; /* Seconds */
			cc=(unsigned char)abs(cc-100);
			yy=(unsigned char)abs(yy-100);
			sprintf(dat,"%d-%s-%d",dd,month[mm-1],(unsigned)cc*100+(unsigned)yy);
			str->assign(dat);
			free(dat);
			retVal=CELL_OK;
			break;
	}
	return retVal;
}

int Cursor::getInt(int col) {
	if(cell[col].isNull==-1) return 0;
	if(head[col].getReqType()==SQLT_STR || head[col].getReqType()==SQLT_LNG) return atoi((char *)cell[col].data);
	if(head[col].getReqType()==SQLT_DAT) throw(new OraError("Error - Cannot convert date to int\n",-1));
}

double Cursor::getDouble(int col) {
	if(cell[col].isNull==-1) return 0;
	if(head[col].getReqType()==SQLT_STR || head[col].getReqType()==SQLT_LNG) return atof((char *)cell[col].data);
	if(head[col].getReqType()==SQLT_DAT) throw(new OraError("Error - Cannot convert date to double\n",-1));
}

sb4 Cursor::getNCols() {
	return nCols;
}	

text *Cursor::getColName(int col) {
	if(canFetch) return (text *)head[col].getAttrName();
	else return 0;
}
int Cursor::getColSize(int col) {
	if(canFetch) return (int)head[col].getAttrSize();
	else return 0;
}
/* Set's text which will be returned on request of null cell default "" (empty text) */	
void Cursor::setNullText(const char *nulltext) {
	this->nulltext=(char *)calloc(strlen(nulltext)+1,1);
	strcpy(this->nulltext,nulltext);
}

void Cursor::correctTypes() {
	for(int i=0;i<nCols;i++) {
		if(head[i].getAttrType()==SQLT_NUM) {
			head[i].setReqType(SQLT_STR);
			if(!head[i].getAttrSize()) { /* Precision is 0 choose our own len */
				head[i].setAttrSize(255); /* hope it's enough to keep most of oracle numbers */
			} else { 
				head[i].setAttrSize(head[i].getAttrSize()+1);
			}
		} else if(head[i].getAttrType()==SQLT_LNG) {
			head[i].setAttrSize(4000);
			head[i].setReqType(SQLT_STR);
		} else if(head[i].getAttrType()==SQLT_INT) {
			head[i].setAttrSize(12); /* Just to set host language int size on [LW]Intelx86 arcs should be 4 */
			head[i].setReqType(SQLT_STR);
		} else if(head[i].getAttrType()==SQLT_AFC) {
			head[i].setReqType(SQLT_AVC);
			head[i].setAttrSize(head[i].getAttrSize()+1);
		} else if(head[i].getAttrType()==SQLT_CHR) {
			head[i].setReqType(SQLT_STR);
			head[i].setAttrSize(head[i].getAttrSize()+1);
		} else {
			head[i].setReqType(head[i].getAttrType());
		}
	} 		
}